// A Bison parser, made by GNU Bison 3.8.2.

// Skeleton implementation for Bison LALR(1) parsers in C++

// Copyright (C) 2002-2015, 2018-2021 Free Software Foundation, Inc.

// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.

// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.

// You should have received a copy of the GNU General Public License
// along with this program.  If not, see <https://www.gnu.org/licenses/>.

// As a special exception, you may create a larger work that contains
// part or all of the Bison parser skeleton and distribute that work
// under terms of your choice, so long as that work isn't itself a
// parser generator using the skeleton or a modified version thereof
// as a parser skeleton.  Alternatively, if you modify or redistribute
// the parser skeleton itself, you may (at your option) remove this
// special exception, which will cause the skeleton and the resulting
// Bison output files to be licensed under the GNU General Public
// License without this special exception.

// This special exception was added by the Free Software Foundation in
// version 2.2 of Bison.

// DO NOT RELY ON FEATURES THAT ARE NOT DOCUMENTED in the manual,
// especially those whose name start with YY_ or yy_.  They are
// private implementation details that can be changed or removed.


// Take the name prefix into account.
#define yylex   KHolidayslex

// First part of user prologue.
#line 18 "/build/kholidays/src/kholidays-6.28.0/src/parsers/plan2/holidayparserplan.ypp"
 /*** C/C++ Declarations ***/

#include <QString>


#line 49 "/build/kholidays/src/build/src/holidayparserplan.cpp"


#include "holidayparserplan.hpp"

// Second part of user prologue.
#line 89 "/build/kholidays/src/kholidays-6.28.0/src/parsers/plan2/holidayparserplan.ypp"


#if defined(__GNUC__)
#pragma GCC diagnostic push
#pragma GCC diagnostic ignored "-Wunused-but-set-variable"
#endif

#include "holidayparserdriverplan_p.h"
#include "holidayscannerplan_p.h"

/* this "connects" the bison parser in the driver to the flex scanner class
 * object. it defines the yylex() function call to pull the next token from the
 * current lexer object of the driver context. */
#undef yylex
#define yylex driver.m_scanner->lex


#line 73 "/build/kholidays/src/build/src/holidayparserplan.cpp"



#ifndef YY_
# if defined YYENABLE_NLS && YYENABLE_NLS
#  if ENABLE_NLS
#   include <libintl.h> // FIXME: INFRINGES ON USER NAME SPACE.
#   define YY_(msgid) dgettext ("bison-runtime", msgid)
#  endif
# endif
# ifndef YY_
#  define YY_(msgid) msgid
# endif
#endif


// Whether we are compiled with exception support.
#ifndef YY_EXCEPTIONS
# if defined __GNUC__ && !defined __EXCEPTIONS
#  define YY_EXCEPTIONS 0
# else
#  define YY_EXCEPTIONS 1
# endif
#endif

#define YYRHSLOC(Rhs, K) ((Rhs)[K].location)
/* YYLLOC_DEFAULT -- Set CURRENT to span from RHS[1] to RHS[N].
   If N is 0, then set CURRENT to the empty location which ends
   the previous symbol: RHS[0] (always defined).  */

# ifndef YYLLOC_DEFAULT
#  define YYLLOC_DEFAULT(Current, Rhs, N)                               \
    do                                                                  \
      if (N)                                                            \
        {                                                               \
          (Current).begin  = YYRHSLOC (Rhs, 1).begin;                   \
          (Current).end    = YYRHSLOC (Rhs, N).end;                     \
        }                                                               \
      else                                                              \
        {                                                               \
          (Current).begin = (Current).end = YYRHSLOC (Rhs, 0).end;      \
        }                                                               \
    while (false)
# endif


// Enable debugging if requested.
#if KHOLIDAYSDEBUG

// A pseudo ostream that takes yydebug_ into account.
# define YYCDEBUG if (yydebug_) (*yycdebug_)

# define YY_SYMBOL_PRINT(Title, Symbol)         \
  do {                                          \
    if (yydebug_)                               \
    {                                           \
      *yycdebug_ << Title << ' ';               \
      yy_print_ (*yycdebug_, Symbol);           \
      *yycdebug_ << '\n';                       \
    }                                           \
  } while (false)

# define YY_REDUCE_PRINT(Rule)          \
  do {                                  \
    if (yydebug_)                       \
      yy_reduce_print_ (Rule);          \
  } while (false)

# define YY_STACK_PRINT()               \
  do {                                  \
    if (yydebug_)                       \
      yy_stack_print_ ();                \
  } while (false)

#else // !KHOLIDAYSDEBUG

# define YYCDEBUG if (false) std::cerr
# define YY_SYMBOL_PRINT(Title, Symbol)  YY_USE (Symbol)
# define YY_REDUCE_PRINT(Rule)           static_cast<void> (0)
# define YY_STACK_PRINT()                static_cast<void> (0)

#endif // !KHOLIDAYSDEBUG

#define yyerrok         (yyerrstatus_ = 0)
#define yyclearin       (yyla.clear ())

#define YYACCEPT        goto yyacceptlab
#define YYABORT         goto yyabortlab
#define YYERROR         goto yyerrorlab
#define YYRECOVERING()  (!!yyerrstatus_)

namespace KHolidays {
#line 166 "/build/kholidays/src/build/src/holidayparserplan.cpp"

  /// Build a parser object.
  HolidayParserPlan::HolidayParserPlan (class HolidayParserDriverPlan& driver_yyarg)
#if KHOLIDAYSDEBUG
    : yydebug_ (false),
      yycdebug_ (&std::cerr),
#else
    :
#endif
      driver (driver_yyarg)
  {}

  HolidayParserPlan::~HolidayParserPlan ()
  {}

  HolidayParserPlan::syntax_error::~syntax_error () YY_NOEXCEPT YY_NOTHROW
  {}

  /*---------.
  | symbol.  |
  `---------*/

  // basic_symbol.
  template <typename Base>
  HolidayParserPlan::basic_symbol<Base>::basic_symbol (const basic_symbol& that)
    : Base (that)
    , value (that.value)
    , location (that.location)
  {}


  /// Constructor for valueless symbols.
  template <typename Base>
  HolidayParserPlan::basic_symbol<Base>::basic_symbol (typename Base::kind_type t, YY_MOVE_REF (location_type) l)
    : Base (t)
    , value ()
    , location (l)
  {}

  template <typename Base>
  HolidayParserPlan::basic_symbol<Base>::basic_symbol (typename Base::kind_type t, YY_RVREF (value_type) v, YY_RVREF (location_type) l)
    : Base (t)
    , value (YY_MOVE (v))
    , location (YY_MOVE (l))
  {}


  template <typename Base>
  HolidayParserPlan::symbol_kind_type
  HolidayParserPlan::basic_symbol<Base>::type_get () const YY_NOEXCEPT
  {
    return this->kind ();
  }


  template <typename Base>
  bool
  HolidayParserPlan::basic_symbol<Base>::empty () const YY_NOEXCEPT
  {
    return this->kind () == symbol_kind::S_YYEMPTY;
  }

  template <typename Base>
  void
  HolidayParserPlan::basic_symbol<Base>::move (basic_symbol& s)
  {
    super_type::move (s);
    value = YY_MOVE (s.value);
    location = YY_MOVE (s.location);
  }

  // by_kind.
  HolidayParserPlan::by_kind::by_kind () YY_NOEXCEPT
    : kind_ (symbol_kind::S_YYEMPTY)
  {}

#if 201103L <= YY_CPLUSPLUS
  HolidayParserPlan::by_kind::by_kind (by_kind&& that) YY_NOEXCEPT
    : kind_ (that.kind_)
  {
    that.clear ();
  }
#endif

  HolidayParserPlan::by_kind::by_kind (const by_kind& that) YY_NOEXCEPT
    : kind_ (that.kind_)
  {}

  HolidayParserPlan::by_kind::by_kind (token_kind_type t) YY_NOEXCEPT
    : kind_ (yytranslate_ (t))
  {}



  void
  HolidayParserPlan::by_kind::clear () YY_NOEXCEPT
  {
    kind_ = symbol_kind::S_YYEMPTY;
  }

  void
  HolidayParserPlan::by_kind::move (by_kind& that)
  {
    kind_ = that.kind_;
    that.clear ();
  }

  HolidayParserPlan::symbol_kind_type
  HolidayParserPlan::by_kind::kind () const YY_NOEXCEPT
  {
    return kind_;
  }


  HolidayParserPlan::symbol_kind_type
  HolidayParserPlan::by_kind::type_get () const YY_NOEXCEPT
  {
    return this->kind ();
  }



  // by_state.
  HolidayParserPlan::by_state::by_state () YY_NOEXCEPT
    : state (empty_state)
  {}

  HolidayParserPlan::by_state::by_state (const by_state& that) YY_NOEXCEPT
    : state (that.state)
  {}

  void
  HolidayParserPlan::by_state::clear () YY_NOEXCEPT
  {
    state = empty_state;
  }

  void
  HolidayParserPlan::by_state::move (by_state& that)
  {
    state = that.state;
    that.clear ();
  }

  HolidayParserPlan::by_state::by_state (state_type s) YY_NOEXCEPT
    : state (s)
  {}

  HolidayParserPlan::symbol_kind_type
  HolidayParserPlan::by_state::kind () const YY_NOEXCEPT
  {
    if (state == empty_state)
      return symbol_kind::S_YYEMPTY;
    else
      return YY_CAST (symbol_kind_type, yystos_[+state]);
  }

  HolidayParserPlan::stack_symbol_type::stack_symbol_type ()
  {}

  HolidayParserPlan::stack_symbol_type::stack_symbol_type (YY_RVREF (stack_symbol_type) that)
    : super_type (YY_MOVE (that.state), YY_MOVE (that.value), YY_MOVE (that.location))
  {
#if 201103L <= YY_CPLUSPLUS
    // that is emptied.
    that.state = empty_state;
#endif
  }

  HolidayParserPlan::stack_symbol_type::stack_symbol_type (state_type s, YY_MOVE_REF (symbol_type) that)
    : super_type (s, YY_MOVE (that.value), YY_MOVE (that.location))
  {
    // that is emptied.
    that.kind_ = symbol_kind::S_YYEMPTY;
  }

#if YY_CPLUSPLUS < 201103L
  HolidayParserPlan::stack_symbol_type&
  HolidayParserPlan::stack_symbol_type::operator= (const stack_symbol_type& that)
  {
    state = that.state;
    value = that.value;
    location = that.location;
    return *this;
  }

  HolidayParserPlan::stack_symbol_type&
  HolidayParserPlan::stack_symbol_type::operator= (stack_symbol_type& that)
  {
    state = that.state;
    value = that.value;
    location = that.location;
    // that is emptied.
    that.state = empty_state;
    return *this;
  }
#endif

  template <typename Base>
  void
  HolidayParserPlan::yy_destroy_ (const char* yymsg, basic_symbol<Base>& yysym) const
  {
    if (yymsg)
      YY_SYMBOL_PRINT (yymsg, yysym);

    // User destructor.
    switch (yysym.kind ())
    {
      case symbol_kind::S_STRING: // STRING
#line 74 "/build/kholidays/src/kholidays-6.28.0/src/parsers/plan2/holidayparserplan.ypp"
                    { free( (yysym.value.sval) ); }
#line 378 "/build/kholidays/src/build/src/holidayparserplan.cpp"
        break;

      default:
        break;
    }
  }

#if KHOLIDAYSDEBUG
  template <typename Base>
  void
  HolidayParserPlan::yy_print_ (std::ostream& yyo, const basic_symbol<Base>& yysym) const
  {
    std::ostream& yyoutput = yyo;
    YY_USE (yyoutput);
    if (yysym.empty ())
      yyo << "empty symbol";
    else
      {
        symbol_kind_type yykind = yysym.kind ();
        yyo << (yykind < YYNTOKENS ? "token" : "nterm")
            << ' ' << yysym.name () << " ("
            << yysym.location << ": ";
        YY_USE (yykind);
        yyo << ')';
      }
  }
#endif

  void
  HolidayParserPlan::yypush_ (const char* m, YY_MOVE_REF (stack_symbol_type) sym)
  {
    if (m)
      YY_SYMBOL_PRINT (m, sym);
    yystack_.push (YY_MOVE (sym));
  }

  void
  HolidayParserPlan::yypush_ (const char* m, state_type s, YY_MOVE_REF (symbol_type) sym)
  {
#if 201103L <= YY_CPLUSPLUS
    yypush_ (m, stack_symbol_type (s, std::move (sym)));
#else
    stack_symbol_type ss (s, sym);
    yypush_ (m, ss);
#endif
  }

  void
  HolidayParserPlan::yypop_ (int n) YY_NOEXCEPT
  {
    yystack_.pop (n);
  }

#if KHOLIDAYSDEBUG
  std::ostream&
  HolidayParserPlan::debug_stream () const
  {
    return *yycdebug_;
  }

  void
  HolidayParserPlan::set_debug_stream (std::ostream& o)
  {
    yycdebug_ = &o;
  }


  HolidayParserPlan::debug_level_type
  HolidayParserPlan::debug_level () const
  {
    return yydebug_;
  }

  void
  HolidayParserPlan::set_debug_level (debug_level_type l)
  {
    yydebug_ = l;
  }
#endif // KHOLIDAYSDEBUG

  HolidayParserPlan::state_type
  HolidayParserPlan::yy_lr_goto_state_ (state_type yystate, int yysym)
  {
    int yyr = yypgoto_[yysym - YYNTOKENS] + yystate;
    if (0 <= yyr && yyr <= yylast_ && yycheck_[yyr] == yystate)
      return yytable_[yyr];
    else
      return yydefgoto_[yysym - YYNTOKENS];
  }

  bool
  HolidayParserPlan::yy_pact_value_is_default_ (int yyvalue) YY_NOEXCEPT
  {
    return yyvalue == yypact_ninf_;
  }

  bool
  HolidayParserPlan::yy_table_value_is_error_ (int yyvalue) YY_NOEXCEPT
  {
    return yyvalue == yytable_ninf_;
  }

  int
  HolidayParserPlan::operator() ()
  {
    return parse ();
  }

  int
  HolidayParserPlan::parse ()
  {
    int yyn;
    /// Length of the RHS of the rule being reduced.
    int yylen = 0;

    // Error handling.
    int yynerrs_ = 0;
    int yyerrstatus_ = 0;

    /// The lookahead symbol.
    symbol_type yyla;

    /// The locations where the error started and ended.
    stack_symbol_type yyerror_range[3];

    /// The return value of parse ().
    int yyresult;

#if YY_EXCEPTIONS
    try
#endif // YY_EXCEPTIONS
      {
    YYCDEBUG << "Starting parse\n";


    // User initialization code.
#line 48 "/build/kholidays/src/kholidays-6.28.0/src/parsers/plan2/holidayparserplan.ypp"
{
    // initialize the initial location object
    yyla.location.begin.filename = driver.fileToParse();
    yyla.location.end.filename = yyla.location.begin.filename;
}

#line 522 "/build/kholidays/src/build/src/holidayparserplan.cpp"


    /* Initialize the stack.  The initial state will be set in
       yynewstate, since the latter expects the semantical and the
       location values to have been already stored, initialize these
       stacks with a primary value.  */
    yystack_.clear ();
    yypush_ (YY_NULLPTR, 0, YY_MOVE (yyla));

  /*-----------------------------------------------.
  | yynewstate -- push a new symbol on the stack.  |
  `-----------------------------------------------*/
  yynewstate:
    YYCDEBUG << "Entering state " << int (yystack_[0].state) << '\n';
    YY_STACK_PRINT ();

    // Accept?
    if (yystack_[0].state == yyfinal_)
      YYACCEPT;

    goto yybackup;


  /*-----------.
  | yybackup.  |
  `-----------*/
  yybackup:
    // Try to take a decision without lookahead.
    yyn = yypact_[+yystack_[0].state];
    if (yy_pact_value_is_default_ (yyn))
      goto yydefault;

    // Read a lookahead token.
    if (yyla.empty ())
      {
        YYCDEBUG << "Reading a token\n";
#if YY_EXCEPTIONS
        try
#endif // YY_EXCEPTIONS
          {
            yyla.kind_ = yytranslate_ (yylex (&yyla.value, &yyla.location));
          }
#if YY_EXCEPTIONS
        catch (const syntax_error& yyexc)
          {
            YYCDEBUG << "Caught exception: " << yyexc.what() << '\n';
            error (yyexc);
            goto yyerrlab1;
          }
#endif // YY_EXCEPTIONS
      }
    YY_SYMBOL_PRINT ("Next token is", yyla);

    if (yyla.kind () == symbol_kind::S_YYerror)
    {
      // The scanner already issued an error message, process directly
      // to error recovery.  But do not keep the error token as
      // lookahead, it is too special and may lead us to an endless
      // loop in error recovery. */
      yyla.kind_ = symbol_kind::S_YYUNDEF;
      goto yyerrlab1;
    }

    /* If the proper action on seeing token YYLA.TYPE is to reduce or
       to detect an error, take that action.  */
    yyn += yyla.kind ();
    if (yyn < 0 || yylast_ < yyn || yycheck_[yyn] != yyla.kind ())
      {
        goto yydefault;
      }

    // Reduce or error.
    yyn = yytable_[yyn];
    if (yyn <= 0)
      {
        if (yy_table_value_is_error_ (yyn))
          goto yyerrlab;
        yyn = -yyn;
        goto yyreduce;
      }

    // Count tokens shifted since error; after three, turn off error status.
    if (yyerrstatus_)
      --yyerrstatus_;

    // Shift the lookahead token.
    yypush_ ("Shifting", state_type (yyn), YY_MOVE (yyla));
    goto yynewstate;


  /*-----------------------------------------------------------.
  | yydefault -- do the default action for the current state.  |
  `-----------------------------------------------------------*/
  yydefault:
    yyn = yydefact_[+yystack_[0].state];
    if (yyn == 0)
      goto yyerrlab;
    goto yyreduce;


  /*-----------------------------.
  | yyreduce -- do a reduction.  |
  `-----------------------------*/
  yyreduce:
    yylen = yyr2_[yyn];
    {
      stack_symbol_type yylhs;
      yylhs.state = yy_lr_goto_state_ (yystack_[yylen].state, yyr1_[yyn]);
      /* If YYLEN is nonzero, implement the default value of the
         action: '$$ = $1'.  Otherwise, use the top of the stack.

         Otherwise, the following line sets YYLHS.VALUE to garbage.
         This behavior is undocumented and Bison users should not rely
         upon it.  */
      if (yylen)
        yylhs.value = yystack_[yylen - 1].value;
      else
        yylhs.value = yystack_[0].value;

      // Default location.
      {
        stack_type::slice range (yystack_, yylen);
        YYLLOC_DEFAULT (yylhs.location, range, yylen);
        yyerror_range[1].location = yylhs.location;
      }

      // Perform the reduction.
      YY_REDUCE_PRINT (yyn);
#if YY_EXCEPTIONS
      try
#endif // YY_EXCEPTIONS
        {
          switch (yyn)
            {
  case 4: // countrycode: %empty
#line 115 "/build/kholidays/src/kholidays-6.28.0/src/parsers/plan2/holidayparserplan.ypp"
                             { driver.setFileCountryCode( QString() ); }
#line 660 "/build/kholidays/src/build/src/holidayparserplan.cpp"
    break;

  case 5: // countrycode: COUNTRY STRING
#line 116 "/build/kholidays/src/kholidays-6.28.0/src/parsers/plan2/holidayparserplan.ypp"
                             { char *s = (yystack_[0].value.sval); driver.setFileCountryCode( QString::fromUtf8( s ) ); free( s ); (yystack_[0].value.sval) = NULL; }
#line 666 "/build/kholidays/src/build/src/holidayparserplan.cpp"
    break;

  case 6: // languagecode: %empty
#line 119 "/build/kholidays/src/kholidays-6.28.0/src/parsers/plan2/holidayparserplan.ypp"
                               { driver.setFileLanguageCode( QString() ); }
#line 672 "/build/kholidays/src/build/src/holidayparserplan.cpp"
    break;

  case 7: // languagecode: LANGUAGE STRING
#line 120 "/build/kholidays/src/kholidays-6.28.0/src/parsers/plan2/holidayparserplan.ypp"
                               { char *s = (yystack_[0].value.sval); driver.setFileLanguageCode( QString::fromUtf8( s ) ); free( s ); (yystack_[0].value.sval) = NULL; }
#line 678 "/build/kholidays/src/build/src/holidayparserplan.cpp"
    break;

  case 8: // name: %empty
#line 123 "/build/kholidays/src/kholidays-6.28.0/src/parsers/plan2/holidayparserplan.ypp"
                   { driver.setFileName( QString() ); }
#line 684 "/build/kholidays/src/build/src/holidayparserplan.cpp"
    break;

  case 9: // name: NAME STRING
#line 124 "/build/kholidays/src/kholidays-6.28.0/src/parsers/plan2/holidayparserplan.ypp"
                   { char *s = (yystack_[0].value.sval); driver.setFileName( QString::fromUtf8( s ) ); free( s ); (yystack_[0].value.sval) = NULL; }
#line 690 "/build/kholidays/src/build/src/holidayparserplan.cpp"
    break;

  case 10: // description: %empty
#line 127 "/build/kholidays/src/kholidays-6.28.0/src/parsers/plan2/holidayparserplan.ypp"
                                 { driver.setFileDescription( QString() ); }
#line 696 "/build/kholidays/src/build/src/holidayparserplan.cpp"
    break;

  case 11: // description: DESCRIPTION STRING
#line 128 "/build/kholidays/src/kholidays-6.28.0/src/parsers/plan2/holidayparserplan.ypp"
                                 { char *s = (yystack_[0].value.sval); driver.setFileDescription( QString::fromUtf8( s ) ); free( s ); (yystack_[0].value.sval) = NULL; }
#line 702 "/build/kholidays/src/build/src/holidayparserplan.cpp"
    break;

  case 14: // eventname: STRING
#line 135 "/build/kholidays/src/kholidays-6.28.0/src/parsers/plan2/holidayparserplan.ypp"
                   { char *s = (yystack_[0].value.sval); driver.setEventName( QString::fromUtf8( s ) ); free( s ); (yystack_[0].value.sval) = NULL; }
#line 708 "/build/kholidays/src/build/src/holidayparserplan.cpp"
    break;

  case 15: // categories: CATEGORY
#line 138 "/build/kholidays/src/kholidays-6.28.0/src/parsers/plan2/holidayparserplan.ypp"
                      { driver.setEventCategory( (yystack_[0].value.sval) ); }
#line 714 "/build/kholidays/src/build/src/holidayparserplan.cpp"
    break;

  case 16: // categories: categories CATEGORY
#line 139 "/build/kholidays/src/kholidays-6.28.0/src/parsers/plan2/holidayparserplan.ypp"
                                 { driver.setEventCategory( (yystack_[1].value.sval) ); }
#line 720 "/build/kholidays/src/build/src/holidayparserplan.cpp"
    break;

  case 17: // calendar: %empty
#line 142 "/build/kholidays/src/kholidays-6.28.0/src/parsers/plan2/holidayparserplan.ypp"
                     { driver.setEventCalendarType( QLatin1String("gregorian") ); }
#line 726 "/build/kholidays/src/build/src/holidayparserplan.cpp"
    break;

  case 18: // calendar: CALENDAR
#line 143 "/build/kholidays/src/kholidays-6.28.0/src/parsers/plan2/holidayparserplan.ypp"
                     { driver.setEventCalendarType( QString::fromUtf8( (yystack_[0].value.sval) ) ); }
#line 732 "/build/kholidays/src/build/src/holidayparserplan.cpp"
    break;

  case 19: // eventrule: EASTER offset length
#line 146 "/build/kholidays/src/kholidays-6.28.0/src/parsers/plan2/holidayparserplan.ypp"
                                                  { driver.setFromEaster( (yystack_[1].value.ival), (yystack_[0].value.ival) ); }
#line 738 "/build/kholidays/src/build/src/holidayparserplan.cpp"
    break;

  case 20: // eventrule: PASCHA offset length
#line 147 "/build/kholidays/src/kholidays-6.28.0/src/parsers/plan2/holidayparserplan.ypp"
                                                  { driver.setFromPascha( (yystack_[1].value.ival), (yystack_[0].value.ival) ); }
#line 744 "/build/kholidays/src/build/src/holidayparserplan.cpp"
    break;

  case 21: // eventrule: date offset conditionaloffset length
#line 148 "/build/kholidays/src/kholidays-6.28.0/src/parsers/plan2/holidayparserplan.ypp"
                                                  { driver.setFromDate( (yystack_[2].value.ival), (yystack_[1].value.ival), (yystack_[0].value.ival) ); }
#line 750 "/build/kholidays/src/build/src/holidayparserplan.cpp"
    break;

  case 22: // eventrule: WDAY offset length
#line 149 "/build/kholidays/src/kholidays-6.28.0/src/parsers/plan2/holidayparserplan.ypp"
                                                  { driver.setFromWeekdayInMonth( 1, (yystack_[2].value.ival), 1, (yystack_[1].value.ival), (yystack_[0].value.ival) ); }
#line 756 "/build/kholidays/src/build/src/holidayparserplan.cpp"
    break;

  case 23: // eventrule: pexpr WDAY offset length
#line 150 "/build/kholidays/src/kholidays-6.28.0/src/parsers/plan2/holidayparserplan.ypp"
                                                  { driver.setFromWeekdayInMonth( (yystack_[3].value.ival), (yystack_[2].value.ival), 1, (yystack_[1].value.ival), (yystack_[0].value.ival) ); }
#line 762 "/build/kholidays/src/build/src/holidayparserplan.cpp"
    break;

  case 24: // eventrule: pexpr WDAY INOP month offset length
#line 151 "/build/kholidays/src/kholidays-6.28.0/src/parsers/plan2/holidayparserplan.ypp"
                                                  { driver.setFromWeekdayInMonth( (yystack_[5].value.ival), (yystack_[4].value.ival), (yystack_[2].value.ival), (yystack_[1].value.ival), (yystack_[0].value.ival) ); }
#line 768 "/build/kholidays/src/build/src/holidayparserplan.cpp"
    break;

  case 25: // eventrule: WDAY pexpr date offset length
#line 152 "/build/kholidays/src/kholidays-6.28.0/src/parsers/plan2/holidayparserplan.ypp"
                                                  { driver.setFromRelativeWeekday( (yystack_[3].value.ival), (yystack_[4].value.ival), (yystack_[1].value.ival), (yystack_[0].value.ival) ); }
#line 774 "/build/kholidays/src/build/src/holidayparserplan.cpp"
    break;

  case 26: // offset: %empty
#line 155 "/build/kholidays/src/kholidays-6.28.0/src/parsers/plan2/holidayparserplan.ypp"
                     { (yylhs.value.ival) =   0; }
#line 780 "/build/kholidays/src/build/src/holidayparserplan.cpp"
    break;

  case 27: // offset: PLUS expr
#line 156 "/build/kholidays/src/kholidays-6.28.0/src/parsers/plan2/holidayparserplan.ypp"
                     { (yylhs.value.ival) =  (yystack_[0].value.ival); }
#line 786 "/build/kholidays/src/build/src/holidayparserplan.cpp"
    break;

  case 28: // offset: MINUS expr
#line 157 "/build/kholidays/src/kholidays-6.28.0/src/parsers/plan2/holidayparserplan.ypp"
                     { (yylhs.value.ival) = -(yystack_[0].value.ival); }
#line 792 "/build/kholidays/src/build/src/holidayparserplan.cpp"
    break;

  case 29: // conditionaloffset: %empty
#line 160 "/build/kholidays/src/kholidays-6.28.0/src/parsers/plan2/holidayparserplan.ypp"
                                                          { (yylhs.value.ival) = 0; }
#line 798 "/build/kholidays/src/build/src/holidayparserplan.cpp"
    break;

  case 30: // conditionaloffset: SHIFT wdaycondition IF wdaycondition
#line 161 "/build/kholidays/src/kholidays-6.28.0/src/parsers/plan2/holidayparserplan.ypp"
                                                          { (yylhs.value.ival) = ( (yystack_[2].value.ival) << 8 ) | (yystack_[0].value.ival); }
#line 804 "/build/kholidays/src/build/src/holidayparserplan.cpp"
    break;

  case 31: // wdaycondition: %empty
#line 164 "/build/kholidays/src/kholidays-6.28.0/src/parsers/plan2/holidayparserplan.ypp"
                                       { (yylhs.value.ival) = 0; }
#line 810 "/build/kholidays/src/build/src/holidayparserplan.cpp"
    break;

  case 32: // wdaycondition: WDAY
#line 165 "/build/kholidays/src/kholidays-6.28.0/src/parsers/plan2/holidayparserplan.ypp"
                                       { (yylhs.value.ival) = ( 1 << (yystack_[0].value.ival) ); }
#line 816 "/build/kholidays/src/build/src/holidayparserplan.cpp"
    break;

  case 33: // wdaycondition: WDAY OR wdaycondition
#line 166 "/build/kholidays/src/kholidays-6.28.0/src/parsers/plan2/holidayparserplan.ypp"
                                       { (yylhs.value.ival) = ( 1 << (yystack_[2].value.ival) ) | (yystack_[0].value.ival); }
#line 822 "/build/kholidays/src/build/src/holidayparserplan.cpp"
    break;

  case 34: // length: %empty
#line 169 "/build/kholidays/src/kholidays-6.28.0/src/parsers/plan2/holidayparserplan.ypp"
                      { (yylhs.value.ival) =  1; }
#line 828 "/build/kholidays/src/build/src/holidayparserplan.cpp"
    break;

  case 35: // length: LENGTH expr
#line 170 "/build/kholidays/src/kholidays-6.28.0/src/parsers/plan2/holidayparserplan.ypp"
                      { (yylhs.value.ival) = (yystack_[0].value.ival); }
#line 834 "/build/kholidays/src/build/src/holidayparserplan.cpp"
    break;

  case 36: // date: pexpr '.' month
#line 173 "/build/kholidays/src/kholidays-6.28.0/src/parsers/plan2/holidayparserplan.ypp"
                                  { driver.setEventDate( -99999, (yystack_[0].value.ival), (yystack_[2].value.ival) ); }
#line 840 "/build/kholidays/src/build/src/holidayparserplan.cpp"
    break;

  case 37: // date: pexpr '.' month '.'
#line 174 "/build/kholidays/src/kholidays-6.28.0/src/parsers/plan2/holidayparserplan.ypp"
                                  { driver.setEventDate( -99999, (yystack_[1].value.ival), (yystack_[3].value.ival) ); }
#line 846 "/build/kholidays/src/build/src/holidayparserplan.cpp"
    break;

  case 38: // date: pexpr '.' month '.' expr
#line 175 "/build/kholidays/src/kholidays-6.28.0/src/parsers/plan2/holidayparserplan.ypp"
                                  { driver.setEventDate(     (yystack_[0].value.ival), (yystack_[2].value.ival), (yystack_[4].value.ival) ); }
#line 852 "/build/kholidays/src/build/src/holidayparserplan.cpp"
    break;

  case 39: // date: month '/' pexpr
#line 176 "/build/kholidays/src/kholidays-6.28.0/src/parsers/plan2/holidayparserplan.ypp"
                                  { driver.setEventDate( -99999, (yystack_[2].value.ival), (yystack_[0].value.ival) ); }
#line 858 "/build/kholidays/src/build/src/holidayparserplan.cpp"
    break;

  case 40: // date: month '/' pexpr '/' pexpr
#line 177 "/build/kholidays/src/kholidays-6.28.0/src/parsers/plan2/holidayparserplan.ypp"
                                  { driver.setEventDate(     (yystack_[0].value.ival), (yystack_[4].value.ival), (yystack_[2].value.ival) ); }
#line 864 "/build/kholidays/src/build/src/holidayparserplan.cpp"
    break;

  case 41: // date: monthnumber pexpr
#line 178 "/build/kholidays/src/kholidays-6.28.0/src/parsers/plan2/holidayparserplan.ypp"
                                  { driver.setEventDate( -99999, (yystack_[1].value.ival), (yystack_[0].value.ival) ); }
#line 870 "/build/kholidays/src/build/src/holidayparserplan.cpp"
    break;

  case 42: // date: monthnumber pexpr pexpr
#line 179 "/build/kholidays/src/kholidays-6.28.0/src/parsers/plan2/holidayparserplan.ypp"
                                  { driver.setEventDate(     (yystack_[0].value.ival), (yystack_[2].value.ival), (yystack_[1].value.ival) ); }
#line 876 "/build/kholidays/src/build/src/holidayparserplan.cpp"
    break;

  case 43: // date: pexpr monthnumber
#line 180 "/build/kholidays/src/kholidays-6.28.0/src/parsers/plan2/holidayparserplan.ypp"
                                  { driver.setEventDate( -99999, (yystack_[0].value.ival), (yystack_[1].value.ival) ); }
#line 882 "/build/kholidays/src/build/src/holidayparserplan.cpp"
    break;

  case 44: // date: pexpr monthnumber pexpr
#line 181 "/build/kholidays/src/kholidays-6.28.0/src/parsers/plan2/holidayparserplan.ypp"
                                  { driver.setEventDate(     (yystack_[0].value.ival), (yystack_[1].value.ival), (yystack_[2].value.ival) ); }
#line 888 "/build/kholidays/src/build/src/holidayparserplan.cpp"
    break;

  case 45: // date: pexpr '.' MONTH pexpr
#line 182 "/build/kholidays/src/kholidays-6.28.0/src/parsers/plan2/holidayparserplan.ypp"
                                  { driver.setEventDate(     (yystack_[0].value.ival), (yystack_[1].value.ival), (yystack_[3].value.ival) ); }
#line 894 "/build/kholidays/src/build/src/holidayparserplan.cpp"
    break;

  case 46: // date: pexpr
#line 183 "/build/kholidays/src/kholidays-6.28.0/src/parsers/plan2/holidayparserplan.ypp"
                                  { driver.setEventDate( (yystack_[0].value.ival) ); }
#line 900 "/build/kholidays/src/build/src/holidayparserplan.cpp"
    break;

  case 47: // reldate: STRING
#line 186 "/build/kholidays/src/kholidays-6.28.0/src/parsers/plan2/holidayparserplan.ypp"
                                 { char *s = (yystack_[0].value.sval); (yylhs.value.ival) = driver.julianDayFromEventName( s ); free( s ); (yystack_[0].value.sval) = NULL; }
#line 906 "/build/kholidays/src/build/src/holidayparserplan.cpp"
    break;

  case 48: // reldate: EASTER
#line 187 "/build/kholidays/src/kholidays-6.28.0/src/parsers/plan2/holidayparserplan.ypp"
                                 { (yylhs.value.ival) = driver.julianDayFromEaster(); }
#line 912 "/build/kholidays/src/build/src/holidayparserplan.cpp"
    break;

  case 49: // reldate: PASCHA
#line 188 "/build/kholidays/src/kholidays-6.28.0/src/parsers/plan2/holidayparserplan.ypp"
                                 { (yylhs.value.ival) = driver.julianDayFromPascha(); }
#line 918 "/build/kholidays/src/build/src/holidayparserplan.cpp"
    break;

  case 50: // reldate: pexpr '.' month
#line 189 "/build/kholidays/src/kholidays-6.28.0/src/parsers/plan2/holidayparserplan.ypp"
                                 { (yylhs.value.ival) = driver.julianDayFromMonthDay( (yystack_[0].value.ival), (yystack_[2].value.ival) ); }
#line 924 "/build/kholidays/src/build/src/holidayparserplan.cpp"
    break;

  case 51: // reldate: pexpr '.' month '.'
#line 190 "/build/kholidays/src/kholidays-6.28.0/src/parsers/plan2/holidayparserplan.ypp"
                                 { (yylhs.value.ival) = driver.julianDayFromMonthDay( (yystack_[1].value.ival), (yystack_[3].value.ival) ); }
#line 930 "/build/kholidays/src/build/src/holidayparserplan.cpp"
    break;

  case 52: // reldate: month '/' pexpr
#line 191 "/build/kholidays/src/kholidays-6.28.0/src/parsers/plan2/holidayparserplan.ypp"
                                 { (yylhs.value.ival) = driver.julianDayFromMonthDay( (yystack_[2].value.ival), (yystack_[0].value.ival) ); }
#line 936 "/build/kholidays/src/build/src/holidayparserplan.cpp"
    break;

  case 53: // reldate: pexpr MONTH
#line 192 "/build/kholidays/src/kholidays-6.28.0/src/parsers/plan2/holidayparserplan.ypp"
                                 { (yylhs.value.ival) = driver.julianDayFromMonthDay( (yystack_[0].value.ival), (yystack_[1].value.ival) ); }
#line 942 "/build/kholidays/src/build/src/holidayparserplan.cpp"
    break;

  case 54: // reldate: monthnumber pexpr
#line 193 "/build/kholidays/src/kholidays-6.28.0/src/parsers/plan2/holidayparserplan.ypp"
                                 { (yylhs.value.ival) = driver.julianDayFromMonthDay( (yystack_[1].value.ival), (yystack_[0].value.ival) ); }
#line 948 "/build/kholidays/src/build/src/holidayparserplan.cpp"
    break;

  case 55: // reldate: WDAY pexpr pexpr
#line 194 "/build/kholidays/src/kholidays-6.28.0/src/parsers/plan2/holidayparserplan.ypp"
                                 { (yylhs.value.ival) = driver.julianDayFromRelativeWeekday( (yystack_[1].value.ival), (yystack_[2].value.ival), (yystack_[0].value.ival) ); }
#line 954 "/build/kholidays/src/build/src/holidayparserplan.cpp"
    break;

  case 56: // reldate: pexpr WDAY INOP month
#line 195 "/build/kholidays/src/kholidays-6.28.0/src/parsers/plan2/holidayparserplan.ypp"
                                 { (yylhs.value.ival) = driver.julianDayFromWeekdayInMonth( (yystack_[3].value.ival), (yystack_[2].value.ival), (yystack_[0].value.ival) ); }
#line 960 "/build/kholidays/src/build/src/holidayparserplan.cpp"
    break;

  case 57: // month: monthnumber
#line 198 "/build/kholidays/src/kholidays-6.28.0/src/parsers/plan2/holidayparserplan.ypp"
        { (yylhs.value.ival) = (yystack_[0].value.ival); }
#line 966 "/build/kholidays/src/build/src/holidayparserplan.cpp"
    break;

  case 58: // month: pexpr
#line 199 "/build/kholidays/src/kholidays-6.28.0/src/parsers/plan2/holidayparserplan.ypp"
                     { (yylhs.value.ival) = driver.adjustedMonthNumber( (yystack_[0].value.ival) ); }
#line 972 "/build/kholidays/src/build/src/holidayparserplan.cpp"
    break;

  case 59: // monthnumber: MONTH
#line 202 "/build/kholidays/src/kholidays-6.28.0/src/parsers/plan2/holidayparserplan.ypp"
                     { (yylhs.value.ival) = driver.adjustedMonthNumber( (yystack_[0].value.ival) ); }
#line 978 "/build/kholidays/src/build/src/holidayparserplan.cpp"
    break;

  case 60: // expr: pexpr
#line 205 "/build/kholidays/src/kholidays-6.28.0/src/parsers/plan2/holidayparserplan.ypp"
                               { (yylhs.value.ival) = (yystack_[0].value.ival); }
#line 984 "/build/kholidays/src/build/src/holidayparserplan.cpp"
    break;

  case 61: // expr: expr OR expr
#line 206 "/build/kholidays/src/kholidays-6.28.0/src/parsers/plan2/holidayparserplan.ypp"
                               { (yylhs.value.ival) = (yystack_[2].value.ival) || (yystack_[0].value.ival); }
#line 990 "/build/kholidays/src/build/src/holidayparserplan.cpp"
    break;

  case 62: // expr: expr AND expr
#line 207 "/build/kholidays/src/kholidays-6.28.0/src/parsers/plan2/holidayparserplan.ypp"
                               { (yylhs.value.ival) = (yystack_[2].value.ival) && (yystack_[0].value.ival); }
#line 996 "/build/kholidays/src/build/src/holidayparserplan.cpp"
    break;

  case 63: // expr: expr EQ expr
#line 208 "/build/kholidays/src/kholidays-6.28.0/src/parsers/plan2/holidayparserplan.ypp"
                               { (yylhs.value.ival) = (yystack_[2].value.ival) == (yystack_[0].value.ival); }
#line 1002 "/build/kholidays/src/build/src/holidayparserplan.cpp"
    break;

  case 64: // expr: expr NE expr
#line 209 "/build/kholidays/src/kholidays-6.28.0/src/parsers/plan2/holidayparserplan.ypp"
                               { (yylhs.value.ival) = (yystack_[2].value.ival) != (yystack_[0].value.ival); }
#line 1008 "/build/kholidays/src/build/src/holidayparserplan.cpp"
    break;

  case 65: // expr: expr LE expr
#line 210 "/build/kholidays/src/kholidays-6.28.0/src/parsers/plan2/holidayparserplan.ypp"
                               { (yylhs.value.ival) = (yystack_[2].value.ival) <= (yystack_[0].value.ival); }
#line 1014 "/build/kholidays/src/build/src/holidayparserplan.cpp"
    break;

  case 66: // expr: expr GE expr
#line 211 "/build/kholidays/src/kholidays-6.28.0/src/parsers/plan2/holidayparserplan.ypp"
                               { (yylhs.value.ival) = (yystack_[2].value.ival) >= (yystack_[0].value.ival); }
#line 1020 "/build/kholidays/src/build/src/holidayparserplan.cpp"
    break;

  case 67: // expr: expr LT expr
#line 212 "/build/kholidays/src/kholidays-6.28.0/src/parsers/plan2/holidayparserplan.ypp"
                               { (yylhs.value.ival) = (yystack_[2].value.ival) <  (yystack_[0].value.ival); }
#line 1026 "/build/kholidays/src/build/src/holidayparserplan.cpp"
    break;

  case 68: // expr: expr GT expr
#line 213 "/build/kholidays/src/kholidays-6.28.0/src/parsers/plan2/holidayparserplan.ypp"
                               { (yylhs.value.ival) = (yystack_[2].value.ival) >  (yystack_[0].value.ival); }
#line 1032 "/build/kholidays/src/build/src/holidayparserplan.cpp"
    break;

  case 69: // expr: expr '+' expr
#line 214 "/build/kholidays/src/kholidays-6.28.0/src/parsers/plan2/holidayparserplan.ypp"
                               { (yylhs.value.ival) = (yystack_[2].value.ival) +  (yystack_[0].value.ival); }
#line 1038 "/build/kholidays/src/build/src/holidayparserplan.cpp"
    break;

  case 70: // expr: expr '-' expr
#line 215 "/build/kholidays/src/kholidays-6.28.0/src/parsers/plan2/holidayparserplan.ypp"
                               { (yylhs.value.ival) = (yystack_[2].value.ival) -  (yystack_[0].value.ival); }
#line 1044 "/build/kholidays/src/build/src/holidayparserplan.cpp"
    break;

  case 71: // expr: expr '*' expr
#line 216 "/build/kholidays/src/kholidays-6.28.0/src/parsers/plan2/holidayparserplan.ypp"
                               { (yylhs.value.ival) = (yystack_[2].value.ival) *  (yystack_[0].value.ival); }
#line 1050 "/build/kholidays/src/build/src/holidayparserplan.cpp"
    break;

  case 72: // expr: expr '/' expr
#line 217 "/build/kholidays/src/kholidays-6.28.0/src/parsers/plan2/holidayparserplan.ypp"
                               { (yylhs.value.ival) = (yystack_[0].value.ival) ?  (yystack_[2].value.ival) / (yystack_[0].value.ival) : 0; }
#line 1056 "/build/kholidays/src/build/src/holidayparserplan.cpp"
    break;

  case 73: // expr: expr '%' expr
#line 218 "/build/kholidays/src/kholidays-6.28.0/src/parsers/plan2/holidayparserplan.ypp"
                               { (yylhs.value.ival) = (yystack_[0].value.ival) ?  (yystack_[2].value.ival) % (yystack_[0].value.ival) : 0; }
#line 1062 "/build/kholidays/src/build/src/holidayparserplan.cpp"
    break;

  case 74: // expr: expr '?' expr ':' expr
#line 219 "/build/kholidays/src/kholidays-6.28.0/src/parsers/plan2/holidayparserplan.ypp"
                               { (yylhs.value.ival) = (yystack_[4].value.ival) ?  (yystack_[2].value.ival) : (yystack_[0].value.ival); }
#line 1068 "/build/kholidays/src/build/src/holidayparserplan.cpp"
    break;

  case 75: // expr: '!' expr
#line 220 "/build/kholidays/src/kholidays-6.28.0/src/parsers/plan2/holidayparserplan.ypp"
                               { (yylhs.value.ival) = !(yystack_[0].value.ival); }
#line 1074 "/build/kholidays/src/build/src/holidayparserplan.cpp"
    break;

  case 76: // expr: '[' reldate ']'
#line 221 "/build/kholidays/src/kholidays-6.28.0/src/parsers/plan2/holidayparserplan.ypp"
                               { (yylhs.value.ival) = (yystack_[1].value.ival); }
#line 1080 "/build/kholidays/src/build/src/holidayparserplan.cpp"
    break;

  case 77: // pexpr: '(' expr ')'
#line 224 "/build/kholidays/src/kholidays-6.28.0/src/parsers/plan2/holidayparserplan.ypp"
                      { (yylhs.value.ival) = (yystack_[1].value.ival); }
#line 1086 "/build/kholidays/src/build/src/holidayparserplan.cpp"
    break;

  case 78: // pexpr: number
#line 225 "/build/kholidays/src/kholidays-6.28.0/src/parsers/plan2/holidayparserplan.ypp"
                      { (yylhs.value.ival) = (yystack_[0].value.ival); }
#line 1092 "/build/kholidays/src/build/src/holidayparserplan.cpp"
    break;

  case 79: // number: NUMBER
#line 228 "/build/kholidays/src/kholidays-6.28.0/src/parsers/plan2/holidayparserplan.ypp"
         { (yylhs.value.ival) = (yystack_[0].value.ival); }
#line 1098 "/build/kholidays/src/build/src/holidayparserplan.cpp"
    break;

  case 80: // number: '-' NUMBER
#line 229 "/build/kholidays/src/kholidays-6.28.0/src/parsers/plan2/holidayparserplan.ypp"
                                  { (yylhs.value.ival) = -(yystack_[0].value.ival); }
#line 1104 "/build/kholidays/src/build/src/holidayparserplan.cpp"
    break;

  case 81: // number: YEAR
#line 230 "/build/kholidays/src/kholidays-6.28.0/src/parsers/plan2/holidayparserplan.ypp"
                                  { (yylhs.value.ival) = driver.parseYear(); }
#line 1110 "/build/kholidays/src/build/src/holidayparserplan.cpp"
    break;

  case 82: // number: LEAPYEAR pexpr
#line 231 "/build/kholidays/src/kholidays-6.28.0/src/parsers/plan2/holidayparserplan.ypp"
                                  { (yylhs.value.ival) = driver.isLeapYear( (yystack_[0].value.ival) ); }
#line 1116 "/build/kholidays/src/build/src/holidayparserplan.cpp"
    break;


#line 1120 "/build/kholidays/src/build/src/holidayparserplan.cpp"

            default:
              break;
            }
        }
#if YY_EXCEPTIONS
      catch (const syntax_error& yyexc)
        {
          YYCDEBUG << "Caught exception: " << yyexc.what() << '\n';
          error (yyexc);
          YYERROR;
        }
#endif // YY_EXCEPTIONS
      YY_SYMBOL_PRINT ("-> $$ =", yylhs);
      yypop_ (yylen);
      yylen = 0;

      // Shift the result of the reduction.
      yypush_ (YY_NULLPTR, YY_MOVE (yylhs));
    }
    goto yynewstate;


  /*--------------------------------------.
  | yyerrlab -- here on detecting error.  |
  `--------------------------------------*/
  yyerrlab:
    // If not already recovering from an error, report this error.
    if (!yyerrstatus_)
      {
        ++yynerrs_;
        context yyctx (*this, yyla);
        std::string msg = yysyntax_error_ (yyctx);
        error (yyla.location, YY_MOVE (msg));
      }


    yyerror_range[1].location = yyla.location;
    if (yyerrstatus_ == 3)
      {
        /* If just tried and failed to reuse lookahead token after an
           error, discard it.  */

        // Return failure if at end of input.
        if (yyla.kind () == symbol_kind::S_YYEOF)
          YYABORT;
        else if (!yyla.empty ())
          {
            yy_destroy_ ("Error: discarding", yyla);
            yyla.clear ();
          }
      }

    // Else will try to reuse lookahead token after shifting the error token.
    goto yyerrlab1;


  /*---------------------------------------------------.
  | yyerrorlab -- error raised explicitly by YYERROR.  |
  `---------------------------------------------------*/
  yyerrorlab:
    /* Pacify compilers when the user code never invokes YYERROR and
       the label yyerrorlab therefore never appears in user code.  */
    if (false)
      YYERROR;

    /* Do not reclaim the symbols of the rule whose action triggered
       this YYERROR.  */
    yypop_ (yylen);
    yylen = 0;
    YY_STACK_PRINT ();
    goto yyerrlab1;


  /*-------------------------------------------------------------.
  | yyerrlab1 -- common code for both syntax error and YYERROR.  |
  `-------------------------------------------------------------*/
  yyerrlab1:
    yyerrstatus_ = 3;   // Each real token shifted decrements this.
    // Pop stack until we find a state that shifts the error token.
    for (;;)
      {
        yyn = yypact_[+yystack_[0].state];
        if (!yy_pact_value_is_default_ (yyn))
          {
            yyn += symbol_kind::S_YYerror;
            if (0 <= yyn && yyn <= yylast_
                && yycheck_[yyn] == symbol_kind::S_YYerror)
              {
                yyn = yytable_[yyn];
                if (0 < yyn)
                  break;
              }
          }

        // Pop the current state because it cannot handle the error token.
        if (yystack_.size () == 1)
          YYABORT;

        yyerror_range[1].location = yystack_[0].location;
        yy_destroy_ ("Error: popping", yystack_[0]);
        yypop_ ();
        YY_STACK_PRINT ();
      }
    {
      stack_symbol_type error_token;

      yyerror_range[2].location = yyla.location;
      YYLLOC_DEFAULT (error_token.location, yyerror_range, 2);

      // Shift the error token.
      error_token.state = state_type (yyn);
      yypush_ ("Shifting", YY_MOVE (error_token));
    }
    goto yynewstate;


  /*-------------------------------------.
  | yyacceptlab -- YYACCEPT comes here.  |
  `-------------------------------------*/
  yyacceptlab:
    yyresult = 0;
    goto yyreturn;


  /*-----------------------------------.
  | yyabortlab -- YYABORT comes here.  |
  `-----------------------------------*/
  yyabortlab:
    yyresult = 1;
    goto yyreturn;


  /*-----------------------------------------------------.
  | yyreturn -- parsing is finished, return the result.  |
  `-----------------------------------------------------*/
  yyreturn:
    if (!yyla.empty ())
      yy_destroy_ ("Cleanup: discarding lookahead", yyla);

    /* Do not reclaim the symbols of the rule whose action triggered
       this YYABORT or YYACCEPT.  */
    yypop_ (yylen);
    YY_STACK_PRINT ();
    while (1 < yystack_.size ())
      {
        yy_destroy_ ("Cleanup: popping", yystack_[0]);
        yypop_ ();
      }

    return yyresult;
  }
#if YY_EXCEPTIONS
    catch (...)
      {
        YYCDEBUG << "Exception caught: cleaning lookahead and stack\n";
        // Do not try to display the values of the reclaimed symbols,
        // as their printers might throw an exception.
        if (!yyla.empty ())
          yy_destroy_ (YY_NULLPTR, yyla);

        while (1 < yystack_.size ())
          {
            yy_destroy_ (YY_NULLPTR, yystack_[0]);
            yypop_ ();
          }
        throw;
      }
#endif // YY_EXCEPTIONS
  }

  void
  HolidayParserPlan::error (const syntax_error& yyexc)
  {
    error (yyexc.location, yyexc.what ());
  }

  /* Return YYSTR after stripping away unnecessary quotes and
     backslashes, so that it's suitable for yyerror.  The heuristic is
     that double-quoting is unnecessary unless the string contains an
     apostrophe, a comma, or backslash (other than backslash-backslash).
     YYSTR is taken from yytname.  */
  std::string
  HolidayParserPlan::yytnamerr_ (const char *yystr)
  {
    if (*yystr == '"')
      {
        std::string yyr;
        char const *yyp = yystr;

        for (;;)
          switch (*++yyp)
            {
            case '\'':
            case ',':
              goto do_not_strip_quotes;

            case '\\':
              if (*++yyp != '\\')
                goto do_not_strip_quotes;
              else
                goto append;

            append:
            default:
              yyr += *yyp;
              break;

            case '"':
              return yyr;
            }
      do_not_strip_quotes: ;
      }

    return yystr;
  }

  std::string
  HolidayParserPlan::symbol_name (symbol_kind_type yysymbol)
  {
    return yytnamerr_ (yytname_[yysymbol]);
  }



  // HolidayParserPlan::context.
  HolidayParserPlan::context::context (const HolidayParserPlan& yyparser, const symbol_type& yyla)
    : yyparser_ (yyparser)
    , yyla_ (yyla)
  {}

  int
  HolidayParserPlan::context::expected_tokens (symbol_kind_type yyarg[], int yyargn) const
  {
    // Actual number of expected tokens
    int yycount = 0;

    const int yyn = yypact_[+yyparser_.yystack_[0].state];
    if (!yy_pact_value_is_default_ (yyn))
      {
        /* Start YYX at -YYN if negative to avoid negative indexes in
           YYCHECK.  In other words, skip the first -YYN actions for
           this state because they are default actions.  */
        const int yyxbegin = yyn < 0 ? -yyn : 0;
        // Stay within bounds of both yycheck and yytname.
        const int yychecklim = yylast_ - yyn + 1;
        const int yyxend = yychecklim < YYNTOKENS ? yychecklim : YYNTOKENS;
        for (int yyx = yyxbegin; yyx < yyxend; ++yyx)
          if (yycheck_[yyx + yyn] == yyx && yyx != symbol_kind::S_YYerror
              && !yy_table_value_is_error_ (yytable_[yyx + yyn]))
            {
              if (!yyarg)
                ++yycount;
              else if (yycount == yyargn)
                return 0;
              else
                yyarg[yycount++] = YY_CAST (symbol_kind_type, yyx);
            }
      }

    if (yyarg && yycount == 0 && 0 < yyargn)
      yyarg[0] = symbol_kind::S_YYEMPTY;
    return yycount;
  }






  int
  HolidayParserPlan::yy_syntax_error_arguments_ (const context& yyctx,
                                                 symbol_kind_type yyarg[], int yyargn) const
  {
    /* There are many possibilities here to consider:
       - If this state is a consistent state with a default action, then
         the only way this function was invoked is if the default action
         is an error action.  In that case, don't check for expected
         tokens because there are none.
       - The only way there can be no lookahead present (in yyla) is
         if this state is a consistent state with a default action.
         Thus, detecting the absence of a lookahead is sufficient to
         determine that there is no unexpected or expected token to
         report.  In that case, just report a simple "syntax error".
       - Don't assume there isn't a lookahead just because this state is
         a consistent state with a default action.  There might have
         been a previous inconsistent state, consistent state with a
         non-default action, or user semantic action that manipulated
         yyla.  (However, yyla is currently not documented for users.)
       - Of course, the expected token list depends on states to have
         correct lookahead information, and it depends on the parser not
         to perform extra reductions after fetching a lookahead from the
         scanner and before detecting a syntax error.  Thus, state merging
         (from LALR or IELR) and default reductions corrupt the expected
         token list.  However, the list is correct for canonical LR with
         one exception: it will still contain any token that will not be
         accepted due to an error action in a later state.
    */

    if (!yyctx.lookahead ().empty ())
      {
        if (yyarg)
          yyarg[0] = yyctx.token ();
        int yyn = yyctx.expected_tokens (yyarg ? yyarg + 1 : yyarg, yyargn - 1);
        return yyn + 1;
      }
    return 0;
  }

  // Generate an error message.
  std::string
  HolidayParserPlan::yysyntax_error_ (const context& yyctx) const
  {
    // Its maximum.
    enum { YYARGS_MAX = 5 };
    // Arguments of yyformat.
    symbol_kind_type yyarg[YYARGS_MAX];
    int yycount = yy_syntax_error_arguments_ (yyctx, yyarg, YYARGS_MAX);

    char const* yyformat = YY_NULLPTR;
    switch (yycount)
      {
#define YYCASE_(N, S)                         \
        case N:                               \
          yyformat = S;                       \
        break
      default: // Avoid compiler warnings.
        YYCASE_ (0, YY_("syntax error"));
        YYCASE_ (1, YY_("syntax error, unexpected %s"));
        YYCASE_ (2, YY_("syntax error, unexpected %s, expecting %s"));
        YYCASE_ (3, YY_("syntax error, unexpected %s, expecting %s or %s"));
        YYCASE_ (4, YY_("syntax error, unexpected %s, expecting %s or %s or %s"));
        YYCASE_ (5, YY_("syntax error, unexpected %s, expecting %s or %s or %s or %s"));
#undef YYCASE_
      }

    std::string yyres;
    // Argument number.
    std::ptrdiff_t yyi = 0;
    for (char const* yyp = yyformat; *yyp; ++yyp)
      if (yyp[0] == '%' && yyp[1] == 's' && yyi < yycount)
        {
          yyres += symbol_name (yyarg[yyi++]);
          ++yyp;
        }
      else
        yyres += *yyp;
    return yyres;
  }


  const signed char HolidayParserPlan::yypact_ninf_ = -70;

  const signed char HolidayParserPlan::yytable_ninf_ = -59;

  const short
  HolidayParserPlan::yypact_[] =
  {
     -24,    -1,    25,   -70,     1,   -70,   -70,    39,    42,    38,
     -70,    65,   -70,    64,    51,   -70,     5,   -70,    75,   -70,
     -70,   -70,   114,   -70,   -70,   -70,   129,   -70,     6,    30,
      30,    84,   134,   -70,    30,    58,     6,    -2,   -70,   134,
     134,    77,    63,   -70,    77,    77,   -70,   134,    85,   160,
     -70,    81,     6,     6,    12,   131,     6,   209,   209,   134,
     -70,    30,    43,   -70,   -70,    62,     6,   -70,   -70,   -70,
      54,    67,     6,     3,   134,   134,   134,   134,   134,   134,
     134,   134,   134,   134,   134,   134,   134,   134,   -70,    98,
      77,    71,   -70,    63,    77,     6,    70,   -70,   -70,   -70,
     209,    77,     6,   -70,     6,   -70,   -70,   102,    63,   251,
     251,   251,   251,   251,   251,   230,   251,   116,   116,    62,
      62,    62,   187,    86,    99,   -70,     6,    30,   -70,   -70,
     134,   -70,   -70,   -70,    63,    80,   134,    98,    98,   -70,
      77,   209,   -70,   -70,   -70,   -70,   -70,   -70
  };

  const signed char
  HolidayParserPlan::yydefact_[] =
  {
       4,     0,     0,    12,     6,     5,     1,     2,     0,     8,
      14,     0,     7,     0,    10,    15,    17,     9,     0,     3,
      16,    18,     0,    11,    79,    59,    26,    81,     0,    26,
      26,     0,     0,    13,    26,     0,    57,    46,    78,     0,
       0,    34,     0,    82,    34,    34,    80,     0,     0,     0,
      60,    29,     0,    41,    26,     0,    43,    27,    28,     0,
      22,    26,    46,    19,    20,    75,     0,    47,    48,    49,
       0,     0,    57,    58,     0,     0,     0,     0,     0,     0,
       0,     0,     0,     0,     0,     0,     0,     0,    77,    31,
      34,    39,    42,     0,    34,    59,    36,    57,    58,    44,
      35,    34,     0,    76,     0,    54,    53,     0,     0,    63,
      64,    65,    66,    67,    68,    61,    62,    70,    69,    71,
      72,    73,     0,    32,     0,    21,     0,    26,    23,    45,
      37,    25,    55,    52,     0,    50,     0,    31,    31,    40,
      34,    38,    56,    51,    74,    33,    30,    24
  };

  const signed char
  HolidayParserPlan::yypgoto_[] =
  {
     -70,   -70,   -70,   -70,   -70,   -70,   -70,   -70,   -70,   -70,
     -70,   -70,   -19,   -70,   -69,   -16,    79,   -70,    -9,   111,
     -23,   -22,   -70
  };

  const signed char
  HolidayParserPlan::yydefgoto_[] =
  {
       0,     2,     3,     4,     9,    14,    19,     7,    11,    16,
      22,    33,    41,    90,   124,    60,    34,    70,    35,    97,
      49,    50,    38
  };

  const short
  HolidayParserPlan::yytable_[] =
  {
      37,     1,    25,    54,    42,     5,    43,   106,   107,    24,
      44,    45,    20,    21,    53,    51,    57,    58,    27,    28,
      62,    93,    39,    40,    65,     6,    73,     8,    63,    64,
      91,    92,   -58,    98,    99,    94,   100,    31,    55,    71,
      39,    40,   101,   108,   102,    10,    96,    25,    12,    32,
     105,   109,   110,   111,   112,   113,   114,   115,   116,   117,
     118,   119,   120,   121,   122,    13,    24,    25,   145,   146,
      17,    98,    15,   129,   125,    27,    28,   -58,   128,    18,
     132,    23,   133,    55,   127,   131,    98,    46,    24,    25,
      66,    67,    52,    59,    31,    89,   103,    27,    28,   135,
      87,   104,    68,   123,   139,   126,    32,   141,   140,    69,
     130,   134,    98,   144,   138,   137,    31,    24,    25,    26,
     143,    61,     0,     0,   147,   142,    27,    28,    32,     0,
       0,    29,    24,    36,    24,    95,     0,    24,    30,    39,
      40,    27,    28,    27,    28,    31,    27,    28,    56,    84,
      85,    86,     0,    36,    87,     0,     0,    32,     0,    72,
      31,     0,    31,     0,     0,    31,     0,     0,     0,     0,
      47,     0,    32,    56,    32,    48,     0,    32,    74,    75,
      76,    77,    78,    79,     0,     0,     0,     0,     0,    80,
      81,    82,    83,    84,    85,    86,     0,     0,    87,     0,
       0,     0,     0,     0,    88,    74,    75,    76,    77,    78,
      79,     0,     0,     0,     0,     0,    80,    81,    82,    83,
      84,    85,    86,     0,     0,    87,   136,    74,    75,    76,
      77,    78,    79,     0,     0,     0,     0,     0,    80,    81,
      82,    83,    84,    85,    86,     0,     0,    87,    74,    75,
      76,    77,    78,    79,     0,     0,     0,     0,     0,     0,
      81,    82,    83,    84,    85,    86,     0,     0,    87,    74,
      75,    76,    77,    78,    79,     0,     0,     0,     0,     0,
       0,     0,    82,    83,    84,    85,    86,     0,     0,    87
  };

  const short
  HolidayParserPlan::yycheck_[] =
  {
      22,    25,     4,     5,    26,     6,    28,     4,     5,     3,
      29,    30,     7,     8,    36,    34,    39,    40,    12,    13,
      42,     9,    10,    11,    47,     0,    48,    26,    44,    45,
      52,    53,    34,    55,    56,    54,    59,    31,    40,    48,
      10,    11,    61,    40,    66,     6,    55,     4,     6,    43,
      72,    74,    75,    76,    77,    78,    79,    80,    81,    82,
      83,    84,    85,    86,    87,    27,     3,     4,   137,   138,
       6,    93,     7,    95,    90,    12,    13,    34,    94,    28,
     102,     6,   104,    40,    93,   101,   108,     3,     3,     4,
       5,     6,    34,    16,    31,    14,    42,    12,    13,   108,
      38,    34,    17,     5,   126,    34,    43,   130,   127,    24,
      40,     9,   134,   136,    15,    29,    31,     3,     4,     5,
      40,    42,    -1,    -1,   140,   134,    12,    13,    43,    -1,
      -1,    17,     3,    22,     3,     4,    -1,     3,    24,    10,
      11,    12,    13,    12,    13,    31,    12,    13,    37,    33,
      34,    35,    -1,    42,    38,    -1,    -1,    43,    -1,    48,
      31,    -1,    31,    -1,    -1,    31,    -1,    -1,    -1,    -1,
      36,    -1,    43,    62,    43,    41,    -1,    43,    18,    19,
      20,    21,    22,    23,    -1,    -1,    -1,    -1,    -1,    29,
      30,    31,    32,    33,    34,    35,    -1,    -1,    38,    -1,
      -1,    -1,    -1,    -1,    44,    18,    19,    20,    21,    22,
      23,    -1,    -1,    -1,    -1,    -1,    29,    30,    31,    32,
      33,    34,    35,    -1,    -1,    38,    39,    18,    19,    20,
      21,    22,    23,    -1,    -1,    -1,    -1,    -1,    29,    30,
      31,    32,    33,    34,    35,    -1,    -1,    38,    18,    19,
      20,    21,    22,    23,    -1,    -1,    -1,    -1,    -1,    -1,
      30,    31,    32,    33,    34,    35,    -1,    -1,    38,    18,
      19,    20,    21,    22,    23,    -1,    -1,    -1,    -1,    -1,
      -1,    -1,    31,    32,    33,    34,    35,    -1,    -1,    38
  };

  const signed char
  HolidayParserPlan::yystos_[] =
  {
       0,    25,    46,    47,    48,     6,     0,    52,    26,    49,
       6,    53,     6,    27,    50,     7,    54,     6,    28,    51,
       7,     8,    55,     6,     3,     4,     5,    12,    13,    17,
      24,    31,    43,    56,    61,    63,    64,    66,    67,    10,
      11,    57,    66,    66,    57,    57,     3,    36,    41,    65,
      66,    57,    34,    66,     5,    40,    64,    65,    65,    16,
      60,    61,    66,    60,    60,    65,     5,     6,    17,    24,
      62,    63,    64,    66,    18,    19,    20,    21,    22,    23,
      29,    30,    31,    32,    33,    34,    35,    38,    44,    14,
      58,    66,    66,     9,    57,     4,    63,    64,    66,    66,
      65,    57,    66,    42,    34,    66,     4,     5,    40,    65,
      65,    65,    65,    65,    65,    65,    65,    65,    65,    65,
      65,    65,    65,     5,    59,    60,    34,    63,    60,    66,
      40,    60,    66,    66,     9,    63,    39,    29,    15,    66,
      57,    65,    63,    40,    65,    59,    59,    60
  };

  const signed char
  HolidayParserPlan::yyr1_[] =
  {
       0,    45,    46,    47,    48,    48,    49,    49,    50,    50,
      51,    51,    52,    52,    53,    54,    54,    55,    55,    56,
      56,    56,    56,    56,    56,    56,    57,    57,    57,    58,
      58,    59,    59,    59,    60,    60,    61,    61,    61,    61,
      61,    61,    61,    61,    61,    61,    61,    62,    62,    62,
      62,    62,    62,    62,    62,    62,    62,    63,    63,    64,
      65,    65,    65,    65,    65,    65,    65,    65,    65,    65,
      65,    65,    65,    65,    65,    65,    65,    66,    66,    67,
      67,    67,    67
  };

  const signed char
  HolidayParserPlan::yyr2_[] =
  {
       0,     2,     2,     4,     0,     2,     0,     2,     0,     2,
       0,     2,     0,     5,     1,     1,     2,     0,     1,     3,
       3,     4,     3,     4,     6,     5,     0,     2,     2,     0,
       4,     0,     1,     3,     0,     2,     3,     4,     5,     3,
       5,     2,     3,     2,     3,     4,     1,     1,     1,     1,
       3,     4,     3,     2,     2,     3,     4,     1,     1,     1,
       1,     3,     3,     3,     3,     3,     3,     3,     3,     3,
       3,     3,     3,     3,     5,     2,     3,     3,     1,     1,
       2,     1,     2
  };


#if KHOLIDAYSDEBUG || 1
  // YYTNAME[SYMBOL-NUM] -- String name of the symbol SYMBOL-NUM.
  // First, the terminals, then, starting at \a YYNTOKENS, nonterminals.
  const char*
  const HolidayParserPlan::yytname_[] =
  {
  "END", "error", "\"invalid token\"", "NUMBER", "MONTH", "WDAY",
  "STRING", "CATEGORY", "CALENDAR", "INOP", "PLUS", "MINUS", "YEAR",
  "LEAPYEAR", "SHIFT", "IF", "LENGTH", "EASTER", "EQ", "NE", "LE", "GE",
  "LT", "GT", "PASCHA", "COUNTRY", "LANGUAGE", "NAME", "DESCRIPTION", "OR",
  "AND", "'-'", "'+'", "'*'", "'/'", "'%'", "'!'", "UMINUS", "'?'", "':'",
  "'.'", "'['", "']'", "'('", "')'", "$accept", "planfile", "metadata",
  "countrycode", "languagecode", "name", "description", "list",
  "eventname", "categories", "calendar", "eventrule", "offset",
  "conditionaloffset", "wdaycondition", "length", "date", "reldate",
  "month", "monthnumber", "expr", "pexpr", "number", YY_NULLPTR
  };
#endif


#if KHOLIDAYSDEBUG
  const unsigned char
  HolidayParserPlan::yyrline_[] =
  {
       0,   109,   109,   112,   115,   116,   119,   120,   123,   124,
     127,   128,   131,   132,   135,   138,   139,   142,   143,   146,
     147,   148,   149,   150,   151,   152,   155,   156,   157,   160,
     161,   164,   165,   166,   169,   170,   173,   174,   175,   176,
     177,   178,   179,   180,   181,   182,   183,   186,   187,   188,
     189,   190,   191,   192,   193,   194,   195,   198,   199,   202,
     205,   206,   207,   208,   209,   210,   211,   212,   213,   214,
     215,   216,   217,   218,   219,   220,   221,   224,   225,   228,
     229,   230,   231
  };

  void
  HolidayParserPlan::yy_stack_print_ () const
  {
    *yycdebug_ << "Stack now";
    for (stack_type::const_iterator
           i = yystack_.begin (),
           i_end = yystack_.end ();
         i != i_end; ++i)
      *yycdebug_ << ' ' << int (i->state);
    *yycdebug_ << '\n';
  }

  void
  HolidayParserPlan::yy_reduce_print_ (int yyrule) const
  {
    int yylno = yyrline_[yyrule];
    int yynrhs = yyr2_[yyrule];
    // Print the symbols being reduced, and their result.
    *yycdebug_ << "Reducing stack by rule " << yyrule - 1
               << " (line " << yylno << "):\n";
    // The symbols being reduced.
    for (int yyi = 0; yyi < yynrhs; yyi++)
      YY_SYMBOL_PRINT ("   $" << yyi + 1 << " =",
                       yystack_[(yynrhs) - (yyi + 1)]);
  }
#endif // KHOLIDAYSDEBUG

  HolidayParserPlan::symbol_kind_type
  HolidayParserPlan::yytranslate_ (int t) YY_NOEXCEPT
  {
    // YYTRANSLATE[TOKEN-NUM] -- Symbol number corresponding to
    // TOKEN-NUM as returned by yylex.
    static
    const signed char
    translate_table[] =
    {
       0,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,    36,     2,     2,     2,    35,     2,     2,
      43,    44,    33,    32,     2,    31,    40,    34,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,    39,     2,
       2,     2,     2,    38,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,    41,     2,    42,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     1,     2,     3,     4,
       5,     6,     7,     8,     9,    10,    11,    12,    13,    14,
      15,    16,    17,    18,    19,    20,    21,    22,    23,    24,
      25,    26,    27,    28,    29,    30,    37
    };
    // Last valid token kind.
    const int code_max = 286;

    if (t <= 0)
      return symbol_kind::S_YYEOF;
    else if (t <= code_max)
      return static_cast <symbol_kind_type> (translate_table[t]);
    else
      return symbol_kind::S_YYUNDEF;
  }

} // KHolidays
#line 1762 "/build/kholidays/src/build/src/holidayparserplan.cpp"

#line 234 "/build/kholidays/src/kholidays-6.28.0/src/parsers/plan2/holidayparserplan.ypp"

#if defined(__GNUC__)
#pragma GCC diagnostic pop
#endif
/*** Private Yacc callbacks and helper functions ***/

void KHolidays::HolidayParserPlan::error( const KHolidays::HolidayParserPlan::location_type &errorLocation, const std::string &errorMessage )
{
    driver.error( errorLocation, errorMessage.c_str() );
}
