/****************************************************************************
** Meta object code from reading C++ file 'lunarphase.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.1)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../../kholidays-6.28.0/src/lunarphase.h"
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'lunarphase.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.1. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN9KHolidays10LunarPhaseE_t {};
} // unnamed namespace

template <> constexpr inline auto KHolidays::LunarPhase::qt_create_metaobjectdata<qt_meta_tag_ZN9KHolidays10LunarPhaseE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "KHolidays::LunarPhase",
        "Phase",
        "NewMoon",
        "FirstQuarter",
        "LastQuarter",
        "FullMoon",
        "None",
        "WaxingCrescent",
        "WaxingGibbous",
        "WaningGibbous",
        "WaningCrescent"
    };

    QtMocHelpers::UintData qt_methods {
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
        // enum 'Phase'
        QtMocHelpers::EnumData<enum Phase>(1, 1, QMC::EnumFlags{}).add({
            {    2, Phase::NewMoon },
            {    3, Phase::FirstQuarter },
            {    4, Phase::LastQuarter },
            {    5, Phase::FullMoon },
            {    6, Phase::None },
            {    7, Phase::WaxingCrescent },
            {    8, Phase::WaxingGibbous },
            {    9, Phase::WaningGibbous },
            {   10, Phase::WaningCrescent },
        }),
    };
    return QtMocHelpers::metaObjectData<LunarPhase, qt_meta_tag_ZN9KHolidays10LunarPhaseE_t>(QMC::PropertyAccessInStaticMetaCall, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject KHolidays::LunarPhase::staticMetaObject = { {
    nullptr,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN9KHolidays10LunarPhaseE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN9KHolidays10LunarPhaseE_t>.data,
    nullptr,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN9KHolidays10LunarPhaseE_t>.metaTypes,
    nullptr
} };

QT_WARNING_POP
